/*
 * File:     ReplaceCVCommand.java
 * Project:  MPI Linguistic Application
 * Date:     31 October 2005
 *
 * Copyright (C) 2001-2005  Max Planck Institute for Psycholinguistics
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package mpi.eudico.client.annotator.commands;

import mpi.eudico.server.corpora.clomimpl.abstr.TranscriptionImpl;
import mpi.eudico.server.corpora.clomimpl.type.ControlledVocabulary;


/**
 * A Command to replace an existing Controlled Vocabulary in a Transcription
 * by a new Controlled Vocabulary.
 *
 * @author Han Sloetjes
 */
public class ReplaceCVCommand implements UndoableCommand {
    private String commandName;

    // receiver
    private TranscriptionImpl transcription;

    /**
 * Creates a new ReplaceCVCommand instance
 *
 * @param name the name of the command
 */
    public ReplaceCVCommand(String name) {
        commandName = name;
    }

    /**
 * To be implemented later
 */
    public void undo() {
    }

    /**
 * To be implemented later
 */
    public void redo() {
    }

    /**
 * Replacing a ControlledVocabulary comes down to deleting the old one  and
 * adding the new one in one (action)command.<br>
 * <b>Note: </b>it is assumed the types and order of the arguments are
 * correct.
 *
 * @param receiver the Transcription
 * @param arguments the arguments:  <ul><li>arg[0] = the old Controlled
 *        Vocabulary (Controlled Vocabulary)</li> <li>arg[1] = the the new
 *        Controlled Vocabulary (Controlled Vocabulary)</li> </ul>
 */
    public void execute(Object receiver, Object[] arguments) {
        transcription = (TranscriptionImpl) receiver;

        ControlledVocabulary conVoc = (ControlledVocabulary) arguments[0];
        ControlledVocabulary newConVoc = (ControlledVocabulary) arguments[1];

        if ((transcription.getControlledVocabulary(conVoc.getName()) != null) &&
                (newConVoc != null)) {
            transcription.removeControlledVocabulary(conVoc);

            transcription.addControlledVocabulary(newConVoc);
        }
    }

    /**
 * Returns the name of the command.
 *
 * @return the name of the command
 */
    public String getName() {
        return commandName;
    }
}
