/*
 * File:     ExportSmilDialog.java
 * Project:  MPI Linguistic Application
 * Date:     31 October 2005
 *
 * Copyright (C) 2001-2005  Max Planck Institute for Psycholinguistics
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package mpi.eudico.client.annotator.export;

import mpi.eudico.client.annotator.ElanLocale;
import mpi.eudico.client.annotator.Selection;

import mpi.eudico.client.util.EAF2SMIL;

import mpi.eudico.server.corpora.clom.Transcription;

import mpi.eudico.server.corpora.clomimpl.abstr.MediaDescriptor;
import mpi.eudico.server.corpora.clomimpl.abstr.TranscriptionImpl;
import mpi.eudico.server.corpora.clomimpl.dobes.EAF22Parser;

import java.awt.Frame;

import java.io.File;
import java.io.IOException;

import java.util.ArrayList;

import javax.swing.ListSelectionModel;


/**
 * Export dialog to export one selected tier into smil format
 *
 * This dialog can work standalone e.g. without ELAN!
 *
 * Created on Mar 29, 2004
 * @author Alexander Klassmann
 * @version April 2004
 */
public class ExportSmilDialog extends AbstractExportDialog {
    /** Holds value of property DOCUMENT ME! */
    final private EAF22Parser parser;

    /** Holds value of property DOCUMENT ME! */
    final private File eafFile;
    private String mediaURL;
    private Selection selection;

    /**
     * Constructor for standalone use
     * @param eafFile
     */
    public ExportSmilDialog(File eafFile) {
        this(null, false, eafFile, null);
    }

    /**
     *
     * @param parent
     * @param modal
     * @param transcription
     * @param selection
     * @throws IOException
     */
    public ExportSmilDialog(Frame parent, boolean modal,
        Transcription transcription, Selection selection)
        throws IOException {
        this(parent, modal,
            new File(((TranscriptionImpl) transcription).getPathName()),
            ((MediaDescriptor) ((TranscriptionImpl) transcription).getMediaDescriptors()
                                .get(0)).mediaURL);
        this.selection = selection;

        if (selection != null) {
            restrictCheckBox.setEnabled(true);
        }
    }

    /**
     *
     * @param parent
     * @param modal
     * @param eafFile
     */
    public ExportSmilDialog(Frame parent, boolean modal, File eafFile,
        String mediaURL) {
        super(parent, modal);
        this.eafFile = eafFile;
        this.mediaURL = mediaURL;

        fileExtension = "smil";
        fileDescription = "*.smil";

        setTitle(ElanLocale.getString("ExportSmilDialog.Title"));
        titleLabel.setText(ElanLocale.getString("ExportSmilDialog.TitleLabel"));

        parser = new EAF22Parser();

        ArrayList tierVector = parser.getTierNames(eafFile.toString());
        tierList.setListData(tierVector.toArray());
        tierList.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
        tierList.setSelectedIndex(0);

        restrictCheckBox.setEnabled(false);

        setExportFile(ExportUtility.getDefaultExportFile(eafFile, fileExtension));

        pack();
    }

    /**
     *
     * @see mpi.eudico.client.annotator.ExportDialog#export()
     */
    protected void export(File exportFile) throws Exception {
        String[] tierNames = new String[tierList.getSelectedValues().length];

        for (int i = 0; i < tierNames.length; i++) {
            tierNames[i] = (String) tierList.getSelectedValues()[i];
        }

        if (restrictCheckBox.isSelected() && (selection != null)) {
            EAF2SMIL.export2SMIL(eafFile, exportFile, tierNames, mediaURL,
                selection.getBeginTime(), selection.getEndTime());
        } else {
            EAF2SMIL.export2SMIL(eafFile, exportFile, tierNames, mediaURL);
        }
    }

    /**
     * Open Dialog; first argument should be URL to eaf-file
     * @param args
     */
    public static void main(String[] args) {
        String eafFileName = null;

        if (args.length > 0) {
            eafFileName = args[0];
        }

        try {
            File eafFile = new File((eafFileName != null) ? eafFileName
                                                          : (System.getProperty(
                        "user.dir") +
                    "/resources/testdata/elan/elan-example2.eaf"));
            ExportSmilDialog dialog = new ExportSmilDialog(eafFile);
            dialog.setVisible(true);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
