/*
 * File:     GridViewerPopupMenu.java
 * Project:  MPI Linguistic Application
 * Date:     31 October 2005
 *
 * Copyright (C) 2001-2005  Max Planck Institute for Psycholinguistics
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package mpi.eudico.client.annotator.grid;

import mpi.eudico.client.annotator.ElanLocale;

import mpi.eudico.client.annotator.export.ExportGridTable;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.util.Enumeration;

import javax.swing.ButtonGroup;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.table.TableColumn;


/**
 * Popupmenu for AnnotationTable; for changing fontSize and Layout of the table
 * Extracted from GridViewer on Jun 30, 2004
 * @author Alexander Klassmann
 * @version Jun 30, 2004
 */
public class GridViewerPopupMenu extends JPopupMenu implements ActionListener {
    /** Holds value of property DOCUMENT ME! */
    final private AnnotationTable table;

    /** Holds value of property DOCUMENT ME! */
    final private JMenu fontMenu;

    /** Holds value of property DOCUMENT ME! */
    final private JMenuItem toggleMenuItem;

    /** Holds value of property DOCUMENT ME! */
    final private JMenuItem exportMenuItem;

    /** Holds value of property DOCUMENT ME! */
    final private ButtonGroup fontSizeBG;
    private int fontSize;

    /**
     * Creates a new GridViewerPopupMenu instance
     *
     * @param table DOCUMENT ME!
     */
    public GridViewerPopupMenu(AnnotationTable table) {
        this.table = table;

        fontSizeBG = new ButtonGroup();
        fontMenu = new JMenu(ElanLocale.getString("Menu.View.FontSize"));

        JRadioButtonMenuItem fontSize8MI = new JRadioButtonMenuItem("8");
        fontSize8MI.setActionCommand("font8");

        if (table.getFont().getSize() == 8) {
            fontSize8MI.setSelected(true);
        }

        fontSize8MI.addActionListener(this);
        fontSizeBG.add(fontSize8MI);
        fontMenu.add(fontSize8MI);

        JRadioButtonMenuItem fontSize9MI = new JRadioButtonMenuItem("9");
        fontSize9MI.setActionCommand("font9");

        if (table.getFont().getSize() == 9) {
            fontSize9MI.setSelected(true);
        }

        fontSize9MI.addActionListener(this);
        fontSizeBG.add(fontSize9MI);
        fontMenu.add(fontSize9MI);

        JRadioButtonMenuItem fontSize10MI = new JRadioButtonMenuItem("10");
        fontSize10MI.setActionCommand("font10");

        if (table.getFont().getSize() == 10) {
            fontSize10MI.setSelected(true);
        }

        fontSize10MI.addActionListener(this);
        fontSizeBG.add(fontSize10MI);
        fontMenu.add(fontSize10MI);

        JRadioButtonMenuItem fontSize12MI = new JRadioButtonMenuItem("12");
        fontSize12MI.setActionCommand("font12");

        if (table.getFont().getSize() == 12) {
            fontSize12MI.setSelected(true);
        }

        fontSize12MI.addActionListener(this);
        fontSizeBG.add(fontSize12MI);
        fontMenu.add(fontSize12MI);

        JRadioButtonMenuItem fontSize14MI = new JRadioButtonMenuItem("14");
        fontSize14MI.setActionCommand("font14");

        if (table.getFont().getSize() == 14) {
            fontSize14MI.setSelected(true);
        }

        fontSize14MI.addActionListener(this);
        fontSizeBG.add(fontSize14MI);
        fontMenu.add(fontSize14MI);

        JRadioButtonMenuItem fontSize16MI = new JRadioButtonMenuItem("16");
        fontSize16MI.setActionCommand("font16");

        if (table.getFont().getSize() == 16) {
            fontSize16MI.setSelected(true);
        }

        fontSize16MI.addActionListener(this);
        fontSizeBG.add(fontSize16MI);
        fontMenu.add(fontSize16MI);

        JRadioButtonMenuItem fontSize18MI = new JRadioButtonMenuItem("18");
        fontSize18MI.setActionCommand("font18");

        if (table.getFont().getSize() == 18) {
            fontSize18MI.setSelected(true);
        }

        fontSize18MI.addActionListener(this);
        fontSizeBG.add(fontSize18MI);
        fontMenu.add(fontSize18MI);

        //add timeformat toggle
        toggleMenuItem = new JMenuItem(ElanLocale.getString(
                    "Menu.Options.TimeFormat"));
        toggleMenuItem.setActionCommand("TOGGLETIMEFORMAT");
        toggleMenuItem.addActionListener(this);

        exportMenuItem = new JMenuItem(ElanLocale.getString(
                    "Frame.GridFrame.ExportTableAsTab"));
        exportMenuItem.setActionCommand("EXPORT");
        exportMenuItem.addActionListener(this);
    }

    /**
     * The menu action event handling.
     *
     * @param e the action event
     */
    public void actionPerformed(ActionEvent e) {
        String actionCommand = e.getActionCommand();

        if (actionCommand.equals("TOGGLETIMEFORMAT")) {
            table.toggleTimeFormat();
        } else if (actionCommand.equals("EXPORT")) {
            ExportGridTable exporter = new ExportGridTable();
            exporter.exportTableAsTabDelimitedText(table);
        } else if (actionCommand.indexOf("font") != -1) {
            int index = actionCommand.indexOf("font") + 4;

            try {
                table.setFontSize(Integer.parseInt(actionCommand.substring(
                            index)));
                table.repaint();
            } catch (Exception ex) {
                //LOG.info("Could not set font size");
            }
        } else {
            table.setColumnVisible(actionCommand,
                ((JCheckBoxMenuItem) e.getSource()).isSelected());
            table.adjustAnnotationColumns();
        }

        table.doLayout();
    }

    private void makeLayout() {
        removeAll();

        for (int i = 0; i < table.getColumnCount(); i++) {
            TableColumn column = table.getColumnModel().getColumn(i);
            String columnName = (String) column.getIdentifier();

            //Column ANNOTATION should not be removable and thus not in selection menu
            if (!columnName.equals(GridViewerTableModel.TIMEPOINT) &&
                    !columnName.equals(GridViewerTableModel.COUNT) &&
                    !columnName.equals(GridViewerTableModel.ANNOTATION)) {
                JMenuItem menuItem = new JCheckBoxMenuItem(column.getHeaderValue()
                                                                 .toString());
                menuItem.setActionCommand(columnName);
                menuItem.setSelected(table.isColumnVisible(columnName));
                menuItem.addActionListener(this);
                add(menuItem);
            }
        }

        addSeparator();
        add(fontMenu);
        addSeparator();
        add(toggleMenuItem);
        add(exportMenuItem);
    }

    /**
     * update Menu when it pops up
     * @see java.awt.Component#setVisible(boolean)
     */
    public void setVisible(boolean b) {
        if (b) {
            makeLayout();
        }

        super.setVisible(b);
    }

    /**
     * Returns the current font size.
     *
     * @return the current font size
     */
    public int getFontSize() {
        return fontSize;
    }

    /**
     * Sets the font size.
     *
     * @param size the new font size
     */
    public void setFontSize(int size) {
        fontSize = size;

        if (fontSizeBG != null) {
            Enumeration en = fontSizeBG.getElements();
            JMenuItem item;
            String value;

            while (en.hasMoreElements()) {
                item = (JMenuItem) en.nextElement();
                value = item.getText();

                try {
                    int v = Integer.parseInt(value);

                    if (v == fontSize) {
                        item.setSelected(true);

                        if (table != null) {
                            table.setFontSize(size);
                            table.repaint();
                        }

                        break;
                    }
                } catch (NumberFormatException nfe) {
                    //// do nothing
                }
            }
        }
    }
}
