/*
 * File:     Constraint.java
 * Project:  MPI Linguistic Application
 * Date:     31 October 2005
 *
 * Copyright (C) 2001-2005  Max Planck Institute for Psycholinguistics
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package mpi.search.content.query.model;

import java.util.HashMap;


/**
 * Created on Aug 18, 2004
 * @author Alexander Klassmann
 * @version Aug 18, 2004
 */
public interface Constraint {
    /** Holds value of property DOCUMENT ME! */
    public final static String TEMPORAL = "Search.Constraint.Temporal";

    /** Holds value of property DOCUMENT ME! */
    public final static String STRUCTURAL = "Search.Constraint.Structural";

    /** Holds value of property DOCUMENT ME! */
    public final static String[] MODES = { STRUCTURAL, TEMPORAL };

    /** Holds value of property DOCUMENT ME! */
    public final static String ANY = "Search.Constraint.Any";

    /** Holds value of property DOCUMENT ME! */
    public final static String NONE = "Search.Constraint.None";

    /** Holds value of property DOCUMENT ME! */
    public final static String[] QUANTIFIERS = new String[] { ANY, NONE };

    /** Holds value of property DOCUMENT ME! */
    public final static String IS_INSIDE = "Search.Constraint.Inside";

    /** Holds value of property DOCUMENT ME! */
    public final static String OVERLAP = "Search.Constraint.Overlap";

    /** Holds value of property DOCUMENT ME! */
    public final static String NOT_INSIDE = "Search.Constraint.NotInside";

    /** Holds value of property DOCUMENT ME! */
    public final static String NO_OVERLAP = "Search.Constraint.NoOverlap";

    /** Holds value of property DOCUMENT ME! */
    public final static String LEFT_OVERLAP = "Search.Constraint.LeftOverlap";

    /** Holds value of property DOCUMENT ME! */
    public final static String RIGHT_OVERLAP = "Search.Constraint.RightOverlap";

    /** Holds value of property DOCUMENT ME! */
    public final static String WITHIN_OVERALL_DISTANCE = "Search.Constraint.WithinOverallDistance";

    /** Holds value of property DOCUMENT ME! */
    public final static String WITHIN_DISTANCE_TO_LEFT_BOUNDARY = "Search.Constraint.WithinLeftDistance";

    /** Holds value of property DOCUMENT ME! */
    public final static String WITHIN_DISTANCE_TO_RIGHT_BOUNDARY = "Search.Constraint.WithinRightDistance";

    /** Holds value of property DOCUMENT ME! */
    public final static String BEFORE_LEFT_DISTANCE = "Search.Constraint.BeforeLeftDistance";

    /** Holds value of property DOCUMENT ME! */
    public final static String AFTER_RIGHT_DISTANCE = "Search.Constraint.AfterRightDistance";

    /** Holds value of property DOCUMENT ME! */
    public final static String[] ANCHOR_CONSTRAINT_TIME_RELATIONS = {
        IS_INSIDE, OVERLAP, NOT_INSIDE, NO_OVERLAP
    };

    /** Holds value of property DOCUMENT ME! */
    public final static String[] DEPENDENT_CONSTRAINT_TIME_RELATIONS = {
        IS_INSIDE, OVERLAP, LEFT_OVERLAP, RIGHT_OVERLAP, WITHIN_OVERALL_DISTANCE,
        WITHIN_DISTANCE_TO_LEFT_BOUNDARY, WITHIN_DISTANCE_TO_RIGHT_BOUNDARY,
        BEFORE_LEFT_DISTANCE, AFTER_RIGHT_DISTANCE
    };

    /** Holds value of property DOCUMENT ME! */
    public final static String regExMetaChars = "\\.^$*+?{}[]()|";

    /**
      * DOCUMENT ME!
      *
      * @param name DOCUMENT ME!
      * @param value DOCUMENT ME!
      */
    public void addAttribute(String name, String value);

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public HashMap getAttributes();

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public String getName();

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public int getNr();

    /**
     * returns Quantifier like ("ANY" or "NONE")
     * @return String
     */
    public String getQuantifier();

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public String getTierName();

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public String getPattern();

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public String getUnit();

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public long getLowerBoundary();

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public String getLowerBoundaryAsString();

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public long getUpperBoundary();

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public String getUpperBoundaryAsString();

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public boolean isRegEx();

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public boolean isCaseSensitive();

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public String getMode();
}
